@@ -1,7 +1,8 @@
 import asyncio
 import logging
 from abc import abstractmethod
-from typing import Any
+from types import TracebackType
+from typing import Any, Self
 
 from griptape_nodes.drivers.storage import StorageBackend
 
@@ -12,6 +13,19 @@ class WorkflowExecutor:
     def __init__(self) -> None:
         self.output: dict | None = None
 
+    async def __aenter__(self) -> Self:
+        """Async context manager entry."""
+        return self
+
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> None:
+        """Async context manager exit."""
+        return
+
     def run(
         self,
         workflow_name: str,
