@@ -1,5 +1,6 @@
 import logging
 import os
+from pathlib import Path
 from urllib.parse import urljoin
 
 import httpx
@@ -14,52 +15,46 @@ class GriptapeCloudStorageDriver(BaseStorageDriver):
 
     def __init__(
         self,
+        workspace_directory: Path,
         *,
         bucket_id: str,
-        base_url: str | None = None,
         api_key: str | None = None,
-        headers: dict | None = None,
         static_files_directory: str | None = None,
+        **kwargs,
     ) -> None:
         """Initialize the GriptapeCloudStorageDriver.
 
         Args:
+            workspace_directory: The base workspace directory path.
             bucket_id: The ID of the bucket to use. Required.
-            base_url: The base URL for the Griptape Cloud API. If not provided, it will be retrieved from the environment variable "GT_CLOUD_BASE_URL" or default to "https://cloud.griptape.ai".
             api_key: The API key for authentication. If not provided, it will be retrieved from the environment variable "GT_CLOUD_API_KEY".
-            headers: Additional headers to include in the requests. If not provided, the default headers will be used.
             static_files_directory: The directory path prefix for static files. If provided, file names will be prefixed with this path.
+            **kwargs: Additional keyword arguments including base_url and headers.
         """
-        self.base_url = (
-            base_url if base_url is not None else os.environ.get("GT_CLOUD_BASE_URL", "https://cloud.griptape.ai")
-        )
+        super().__init__(workspace_directory)
+
+        self.base_url = kwargs.get("base_url") or os.environ.get("GT_CLOUD_BASE_URL", "https://cloud.griptape.ai")
         self.api_key = api_key if api_key is not None else os.environ.get("GT_CLOUD_API_KEY")
-        self.headers = (
-            headers
-            if headers is not None
-            else {
-                "Authorization": f"Bearer {self.api_key}",
-            }
-        )
+        self.headers = kwargs.get("headers") or {"Authorization": f"Bearer {self.api_key}"}
 
         self.bucket_id = bucket_id
         self.static_files_directory = static_files_directory
 
-    def _get_full_file_path(self, file_name: str) -> str:
-        """Get the full file path including the static files directory prefix.
+    def _get_full_file_path(self, path: Path) -> str:
+        """Get the full file path including workspace directory and static files directory prefix.
 
         Args:
-            file_name: The base file name.
+            path: The relative path from the workspace directory.
 
         Returns:
             The full file path with static files directory prefix if configured.
         """
         if self.static_files_directory:
-            return f"{self.static_files_directory}/{file_name}"
-        return file_name
+            return f"{self.static_files_directory}/{path}"
+        return str(path)
 
-    def create_signed_upload_url(self, file_name: str) -> CreateSignedUploadUrlResponse:
-        full_file_path = self._get_full_file_path(file_name)
+    def create_signed_upload_url(self, path: Path) -> CreateSignedUploadUrlResponse:
+        full_file_path = self._get_full_file_path(path)
         self._create_asset(full_file_path)
 
         url = urljoin(self.base_url, f"/api/buckets/{self.bucket_id}/asset-urls/{full_file_path}")
@@ -67,7 +62,7 @@ class GriptapeCloudStorageDriver(BaseStorageDriver):
             response = httpx.post(url, json={"operation": "PUT"}, headers=self.headers)
             response.raise_for_status()
         except httpx.HTTPStatusError as e:
-            msg = f"Failed to create presigned URL for file {file_name}: {e}"
+            msg = f"Failed to create presigned URL for file {path}: {e}"
             logger.error(msg)
             raise RuntimeError(msg) from e
 
@@ -75,14 +70,14 @@ class GriptapeCloudStorageDriver(BaseStorageDriver):
 
         return {"url": response_data["url"], "headers": response_data.get("headers", {}), "method": "PUT"}
 
-    def create_signed_download_url(self, file_name: str) -> str:
-        full_file_path = self._get_full_file_path(file_name)
+    def create_signed_download_url(self, path: Path) -> str:
+        full_file_path = self._get_full_file_path(path)
         url = urljoin(self.base_url, f"/api/buckets/{self.bucket_id}/asset-urls/{full_file_path}")
         try:
             response = httpx.post(url, json={"method": "GET"}, headers=self.headers)
             response.raise_for_status()
         except httpx.HTTPStatusError as e:
-            msg = f"Failed to create presigned URL for file {file_name}: {e}"
+            msg = f"Failed to create presigned URL for file {path}: {e}"
             logger.error(msg)
             raise RuntimeError(msg) from e
 
@@ -190,19 +185,19 @@ class GriptapeCloudStorageDriver(BaseStorageDriver):
 
         return response.json().get("buckets", [])
 
-    def delete_file(self, file_name: str) -> None:
+    def delete_file(self, path: Path) -> None:
         """Delete a file from the bucket.
 
         Args:
-            file_name: The name of the file to delete.
+            path: The path of the file to delete.
         """
-        full_file_path = self._get_full_file_path(file_name)
+        full_file_path = self._get_full_file_path(path)
         url = urljoin(self.base_url, f"/api/buckets/{self.bucket_id}/assets/{full_file_path}")
 
         try:
             response = httpx.delete(url, headers=self.headers)
             response.raise_for_status()
         except httpx.HTTPStatusError as e:
-            msg = f"Failed to delete file {file_name}: {e}"
+            msg = f"Failed to delete file {path}: {e}"
             logger.error(msg)
             raise RuntimeError(msg) from e
