@@ -225,10 +225,12 @@ class GetFlowStateResultSuccess(WorkflowNotAlteredMixin, ResultPayloadSuccess):
     Args:
         control_node: Name of the current control node (if any)
         resolving_node: Name of the node currently being resolved (if any)
+        involved_nodes: Names of nodes that are queued to be executed or have been executed in the current run.
     """
 
-    control_node: str | None
-    resolving_node: str | None
+    control_nodes: list[str] | None
+    resolving_node: list[str] | None
+    involved_nodes: list[str] | None
 
 
 @dataclass
