@@ -37,6 +37,7 @@ from griptape_nodes.retained_mode.events.app_events import (
 )
 from griptape_nodes.retained_mode.events.base_events import (
     GriptapeNodeEvent,
+    ResultDetails,
     ResultPayloadFailure,
 )
 from griptape_nodes.retained_mode.events.flow_events import (
@@ -61,6 +62,7 @@ if TYPE_CHECKING:
     from griptape_nodes.retained_mode.managers.event_manager import EventManager
     from griptape_nodes.retained_mode.managers.flow_manager import FlowManager
     from griptape_nodes.retained_mode.managers.library_manager import LibraryManager
+    from griptape_nodes.retained_mode.managers.model_manager import ModelManager
     from griptape_nodes.retained_mode.managers.node_manager import NodeManager
     from griptape_nodes.retained_mode.managers.object_manager import ObjectManager
     from griptape_nodes.retained_mode.managers.operation_manager import (
@@ -137,6 +139,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
     _flow_manager: FlowManager
     _context_manager: ContextManager
     _library_manager: LibraryManager
+    _model_manager: ModelManager
     _workflow_manager: WorkflowManager
     _workflow_variables_manager: VariablesManager
     _arbitrary_code_exec_manager: ArbitraryCodeExecManager
@@ -159,6 +162,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
         from griptape_nodes.retained_mode.managers.event_manager import EventManager
         from griptape_nodes.retained_mode.managers.flow_manager import FlowManager
         from griptape_nodes.retained_mode.managers.library_manager import LibraryManager
+        from griptape_nodes.retained_mode.managers.model_manager import ModelManager
         from griptape_nodes.retained_mode.managers.node_manager import NodeManager
         from griptape_nodes.retained_mode.managers.object_manager import ObjectManager
         from griptape_nodes.retained_mode.managers.operation_manager import (
@@ -192,6 +196,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
             self._flow_manager = FlowManager(self._event_manager)
             self._context_manager = ContextManager(self._event_manager)
             self._library_manager = LibraryManager(self._event_manager)
+            self._model_manager = ModelManager(self._event_manager)
             self._workflow_manager = WorkflowManager(self._event_manager)
             self._workflow_variables_manager = VariablesManager(self._event_manager)
             self._arbitrary_code_exec_manager = ArbitraryCodeExecManager(self._event_manager)
@@ -252,7 +257,9 @@ class GriptapeNodes(metaclass=SingletonMeta):
                 type(request).__name__,
                 request,
             )
-            return ResultPayloadFailure(exception=e)
+            return ResultPayloadFailure(
+                exception=e, result_details=f"Unhandled exception while processing {type(request).__name__}: {e}"
+            )
         else:
             return result_event.result
 
@@ -276,7 +283,9 @@ class GriptapeNodes(metaclass=SingletonMeta):
                 type(request).__name__,
                 request,
             )
-            return ResultPayloadFailure(exception=e)
+            return ResultPayloadFailure(
+                exception=e, result_details=f"Unhandled exception while processing async {type(request).__name__}: {e}"
+            )
         else:
             return result_event.result
 
@@ -301,6 +310,10 @@ class GriptapeNodes(metaclass=SingletonMeta):
     def LibraryManager(cls) -> LibraryManager:
         return GriptapeNodes.get_instance()._library_manager
 
+    @classmethod
+    def ModelManager(cls) -> ModelManager:
+        return GriptapeNodes.get_instance()._model_manager
+
     @classmethod
     def ObjectManager(cls) -> ObjectManager:
         return GriptapeNodes.get_instance()._object_manager
@@ -421,6 +434,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
                     major=engine_ver.major,
                     minor=engine_ver.minor,
                     patch=engine_ver.patch,
+                    result_details="Engine version retrieved successfully.",
                 )
             details = f"Attempted to get engine version. Failed because version string '{engine_ver}' wasn't in expected major.minor.patch format."
             logger.error(details)
@@ -447,7 +461,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
         await subscribe_to_topic(topic)
         logger.info("Subscribed to new session topic: %s", topic)
 
-        return AppStartSessionResultSuccess(current_session_id)
+        return AppStartSessionResultSuccess(current_session_id, result_details="Session started successfully.")
 
     async def handle_session_end_request(self, _: AppEndSessionRequest) -> ResultPayload:
         from griptape_nodes.app.app import unsubscribe_from_topic
@@ -465,14 +479,19 @@ class GriptapeNodes(metaclass=SingletonMeta):
             unsubscribe_topic = f"sessions/{previous_session_id}/request"
             await unsubscribe_from_topic(unsubscribe_topic)
 
-            return AppEndSessionResultSuccess(session_id=previous_session_id)
+            return AppEndSessionResultSuccess(
+                session_id=previous_session_id, result_details="Session ended successfully."
+            )
         except Exception as err:
             details = f"Failed to end session due to '{err}'."
             logger.error(details)
             return AppEndSessionResultFailure(result_details=details)
 
     def handle_get_session_request(self, _: AppGetSessionRequest) -> ResultPayload:
-        return AppGetSessionResultSuccess(session_id=GriptapeNodes.SessionManager().get_active_session_id())
+        return AppGetSessionResultSuccess(
+            session_id=GriptapeNodes.SessionManager().get_active_session_id(),
+            result_details="Session ID retrieved successfully.",
+        )
 
     def handle_session_heartbeat_request(self, request: SessionHeartbeatRequest) -> ResultPayload:  # noqa: ARG002
         """Handle session heartbeat requests.
@@ -487,8 +506,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
                 return SessionHeartbeatResultFailure(result_details=details)
 
             details = f"Session heartbeat successful for session: {active_session_id}"
-            logger.debug(details)
-            return SessionHeartbeatResultSuccess()
+            return SessionHeartbeatResultSuccess(result_details=details)
         except Exception as err:
             details = f"Failed to handle session heartbeat: {err}"
             logger.error(details)
@@ -509,7 +527,6 @@ class GriptapeNodes(metaclass=SingletonMeta):
             # Get engine name
             engine_name = GriptapeNodes.EngineIdentityManager().get_engine_name()
 
-            logger.debug("Engine heartbeat successful")
             return EngineHeartbeatResultSuccess(
                 heartbeat_id=request.heartbeat_id,
                 engine_version=engine_version,
@@ -517,6 +534,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
                 engine_id=GriptapeNodes.EngineIdentityManager().get_active_engine_id(),
                 session_id=GriptapeNodes.SessionManager().get_active_session_id(),
                 timestamp=datetime.now(tz=UTC).isoformat(),
+                result_details="Engine heartbeat successful",
                 **instance_info,
                 **workflow_info,
             )
@@ -529,8 +547,9 @@ class GriptapeNodes(metaclass=SingletonMeta):
         """Handle requests to get the current engine name."""
         try:
             engine_name = GriptapeNodes.EngineIdentityManager().get_engine_name()
-            logger.debug("Retrieved engine name: %s", engine_name)
-            return GetEngineNameResultSuccess(engine_name=engine_name)
+            return GetEngineNameResultSuccess(
+                engine_name=engine_name, result_details="Engine name retrieved successfully."
+            )
         except Exception as err:
             error_message = f"Failed to get engine name: {err}"
             logger.error(error_message)
@@ -547,8 +566,11 @@ class GriptapeNodes(metaclass=SingletonMeta):
 
             # Set the new engine name
             GriptapeNodes.EngineIdentityManager().set_engine_name(request.engine_name.strip())
-            logger.info("Engine name set to: %s", request.engine_name.strip())
-            return SetEngineNameResultSuccess(engine_name=request.engine_name.strip())
+            details = f"Engine name set to: {request.engine_name.strip()}"
+            return SetEngineNameResultSuccess(
+                engine_name=request.engine_name.strip(),
+                result_details=ResultDetails(message=details, level=logging.INFO),
+            )
 
         except Exception as err:
             error_message = f"Failed to set engine name: {err}"
