@@ -13,7 +13,7 @@ from watchfiles import Change, PythonFilter, watch
 
 from griptape_nodes.drivers.storage.griptape_cloud_storage_driver import GriptapeCloudStorageDriver
 from griptape_nodes.retained_mode.events.app_events import AppInitializationComplete
-from griptape_nodes.retained_mode.events.base_events import AppEvent
+from griptape_nodes.retained_mode.events.base_events import AppEvent, ResultDetails
 from griptape_nodes.retained_mode.events.sync_events import (
     StartSyncAllCloudWorkflowsRequest,
     StartSyncAllCloudWorkflowsResultFailure,
@@ -130,8 +130,13 @@ class SyncManager:
             workflow_files = [file for file in files if file.endswith(".py")]
 
             if not workflow_files:
-                logger.info("No workflow files found in cloud storage")
-                return StartSyncAllCloudWorkflowsResultSuccess(sync_directory=str(sync_dir), total_workflows=0)
+                return StartSyncAllCloudWorkflowsResultSuccess(
+                    sync_directory=str(sync_dir),
+                    total_workflows=0,
+                    result_details=ResultDetails(
+                        message="No workflow files found in cloud storage.", level=logging.INFO
+                    ),
+                )
 
             # Start background sync with unique ID
             sync_task_id = str(uuid.uuid4())
@@ -149,9 +154,9 @@ class SyncManager:
             logger.error(details)
             return StartSyncAllCloudWorkflowsResultFailure(result_details=details)
         else:
-            logger.info("Started background sync for %d workflow files", len(workflow_files))
+            details = f"Started background sync for {len(workflow_files)} workflow files"
             return StartSyncAllCloudWorkflowsResultSuccess(
-                sync_directory=str(sync_dir), total_workflows=len(workflow_files)
+                sync_directory=str(sync_dir), total_workflows=len(workflow_files), result_details=details
             )
 
     def on_app_initialization_complete(self, _payload: AppInitializationComplete) -> None:
@@ -206,7 +211,10 @@ class SyncManager:
             msg = "Cloud storage api_key not configured. Set GT_CLOUD_API_KEY secret."
             raise RuntimeError(msg)
 
+        workspace_directory = Path(self._config_manager.get_config_value("workspace_directory"))
+
         return GriptapeCloudStorageDriver(
+            workspace_directory,
             bucket_id=bucket_id,
             base_url=base_url,
             api_key=api_key,
@@ -229,7 +237,7 @@ class SyncManager:
             sync_dir = self._sync_dir
 
             # Download file content from cloud
-            file_content = storage_driver.download_file(filename)
+            file_content = storage_driver.download_file(Path(filename))
 
             # Write to local sync directory
             local_file_path = sync_dir / filename
@@ -280,7 +288,7 @@ class SyncManager:
 
             # Upload to cloud storage using the upload_file method
             filename = file_path.name
-            storage_driver.upload_file(filename, file_content)
+            storage_driver.upload_file(Path(filename), file_content)
 
             logger.info("Successfully uploaded workflow file to cloud: %s", filename)
 
@@ -298,7 +306,7 @@ class SyncManager:
             filename = file_path.name
 
             # Use the storage driver's delete method
-            storage_driver.delete_file(filename)
+            storage_driver.delete_file(Path(filename))
             logger.info("Successfully deleted workflow file from cloud: %s", filename)
 
         except Exception as e:
@@ -385,7 +393,7 @@ class SyncManager:
         """
         try:
             # Download file content
-            file_content = storage_driver.download_file(file_name)
+            file_content = storage_driver.download_file(Path(file_name))
 
             # Extract just the filename (remove any directory prefixes)
             local_filename = Path(file_name).name
