@@ -10,6 +10,7 @@ from griptape_nodes.retained_mode.events.base_events import (
 )
 from griptape_nodes.retained_mode.events.config_events import (
     GetConfigCategoryRequest,
+    GetConfigSchemaRequest,
     GetConfigValueRequest,
     SetConfigCategoryRequest,
     SetConfigValueRequest,
@@ -1396,6 +1397,24 @@ class RetainedMode:
         result = GriptapeNodes().handle_request(request)
         return result
 
+    @classmethod
+    def get_config_schema(cls) -> ResultPayload:
+        """Gets the JSON schema for the configuration model.
+
+        Returns:
+            ResultPayload: Contains the configuration schema with field types, enums, and validation rules.
+
+        Example:
+            # Get the configuration schema
+            schema_result = cmd.get_config_schema()
+            if isinstance(schema_result, GetConfigSchemaResultSuccess):
+                schema = schema_result.schema
+                # Use schema to render appropriate UI components
+        """
+        request = GetConfigSchemaRequest()
+        result = GriptapeNodes().handle_request(request)
+        return result
+
     @classmethod
     def rename(cls, object_name: str, requested_name: str) -> ResultPayload:
         """Renames a node or flow.
